/*
 * $LastChangedRevision: 1423 $
 * $LastChangedDate:: 2018-03-30 15:57:13 +0200#$
 *
 * This file is part of X2C. http://www.mechatronic-simulation.org/
 *
 * Copyright (c) 2016, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "LNet.h"
#include "VersionInfo.h"
#include "TableStruct.h"
#include "Services.h"
#include "BlockServices.h"
#include "SerialGeneric.h"

#include "X2C.h"
#include "Hardware.h"
#include "Main.h"

/* X2C maintenance structure */
volatile tTableStruct TheTableStruct;
volatile tTableStruct* TableStruct = &TheTableStruct;

/* LNet protocol buffersize & node identifier */
#define LNET_BUFFERSIZE ((uint8)255)
#define LNET_NODEID ((uint8)1)

static tSerial interface;
static tLNet protocol;
static uint8 bufferLNet[LNET_BUFFERSIZE];


/**
 * @brief SysTick_Handler
 *
 * Interrupt Service Routine for system tick counter, calls X2C main control task.
 */
void SysTick_Handler(void)
{
  mainTask();
}


/**
 * @brief Main function.
 *
 * @return The main function will never return due to the never ending loop.
 */
int main(void)
{
    /** - Initialize "integrated monitor":
     *    - configuration of LNet protocol:
     *      - Node-ID: 1
     *      - Buffer size: 255
     *      .
     *    .
     *  - Initialize serial interface
     */
    initTableStruct();
    initSerialGeneric(&interface);
    initSerial(&interface);
    initLNet(&protocol, bufferLNet, LNET_BUFFERSIZE, LNET_NODEID);
    initServiceTable((tProtocol*)&protocol);
    addCoreServices((tProtocol*)&protocol);
    addBlockServices((tProtocol*)&protocol);
    addTableStructProtocol((tProtocol*)&protocol);
    linkSerial((tProtocol*)&protocol, &interface);

	initVersionInfo(TableStruct, (uint16)0x0001);	/* software version 1 */
	TableStruct->DSPState = PRG_LOADED_STATE;

	/** - Initialize X2C */
    X2C_Init();

    /** - Initialize hardware */
	initHardware();

    /** - Never ending loop -> interrupt driven algorithm */
    while (1)
    {
		/* communication procedure */
    	protocol.communicate((tProtocol*)&protocol);
    }
}


/**
 * @brief Main control task.
 *
 * TODO: This task has to be called periodically.
 * Calling rate = 100us
 */
void mainTask(void)
{
    /*------------------------------------------------------------------------*/
    /** - assign inports */
    /*------------------------------------------------------------------------*/
	x2cModel.inports.bPB0 = (!READ_PB0) * INT16_MAX;
	x2cModel.inports.bPB1 = (!READ_PB1) * INT16_MAX;


    /*------------------------------------------------------------------------*/
    /** - update X2C */
    /*------------------------------------------------------------------------*/
	X2C_Update();

    /*------------------------------------------------------------------------*/
    /** - update outports */
    /*------------------------------------------------------------------------*/
    if (*(x2cModel.outports.bLED0) > 0)
    {
        USER_LED0_ON;
    }
    else
    {
        USER_LED0_OFF;
    }

    if (*(x2cModel.outports.bLED1) > 0)
    {
        USER_LED1_ON;
    }
    else
    {
        USER_LED1_OFF;
    }
}
