/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.ZeroHex;
import de.ixxat.jni.CPointer;

/**
 * This value class represents the LIN Capabilities. The LIN Capabilities
 * can be retrieved by calling {@link ILinSocket#GetCapabilities()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class LinCapabilities
{
  // LIN Controller Types
  /** Unknown LIN controller Type */
  public static final short LIN_CTRL_UNKNOWN    = 0;

  // supported features
  /** master mode */
  public static final int LIN_FEATURE_MASTER    = 0x0001;
  /** automatic bitrate detection */
  public static final int LIN_FEATURE_AUTORATE  = 0x0002;
  /** reception of error frames */
  public static final int LIN_FEATURE_ERRFRAME  = 0x0004;
  /** bus load measurement */
  public static final int LIN_FEATURE_BUSLOAD   = 0x0008;

  /** supported features 
   * @see #LIN_FEATURE_MASTER */
  public int    m_dwFeatures    = 0;
  /** clock frequency of the primary counter in Hz */
  public int    m_dwClockFreq   = 0; 
  /** divisor for the message time stamp counter */
  public int    m_dwTscDivisor  = 0;

  
  /**
   * Creates a new LIN Capabilities object.
   * @throws Throwable
   */
  protected LinCapabilities()
  {
  }
  
  /**
   * Creates a new LIN Capabilities object.
   * 
   * @param pLinCapabilities
   *          CPointer to native memory which contains the LINCAPABILITIES
   *          structure.
   * @throws Throwable
   */
  protected LinCapabilities(CPointer pLinCapabilities) throws Throwable
  {
    fromCPointer(pLinCapabilities);
  }
  
  /**
   * Updates a existing LIN Capabilities object.
   * 
   * @param pLinCapabilities
   *          CPointer to native memory which contains the LINCAPABILITIES
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pLinCapabilities) throws Throwable
  {
    // Update LIN Capabilities
    m_dwFeatures    = pLinCapabilities.getInt(0);
    m_dwClockFreq   = pLinCapabilities.getInt(4); 
    m_dwTscDivisor  = pLinCapabilities.getInt(8);
  }

  /**
   * Returns the LIN Capabilities as string.
   * @return The LIN Capabilities as string.
   */
  public String toString()
  {
    String strCapabilities = "";
    
    strCapabilities += "Features: 0x" + ZeroHex.toZeroHexString(m_dwFeatures) + ", ";
    strCapabilities += "ClockFreq: " + m_dwClockFreq + ", ";
    strCapabilities += "TscDivisor: " + m_dwTscDivisor + ", ";
    
    return strCapabilities;
  }

}
